@@ -15,10 +15,10 @@ from compas.files import XML
 from compas.geometry import Transformation
 from compas.robots.resources.basic import _get_file_format
 from compas.robots.resources.basic import _mesh_import
-from compas.utilities import await_callback
 from compas.utilities import geometric_key
 
 LOGGER = logging.getLogger('compas_fab.backends.ros')
+TIMEOUT = 10
 
 __all__ = [
     'RosFileServerLoader',
@@ -119,7 +119,7 @@ class RosFileServerLoader(object):
                 return _read_file(filename)
 
         param = roslibpy.Param(self.ros, parameter_name)
-        urdf = await_callback(param.get)
+        urdf = param.get(timeout=TIMEOUT)
 
         self.robot_name = self._read_robot_name(urdf)
 
@@ -151,7 +151,7 @@ class RosFileServerLoader(object):
                 return _read_file(filename)
 
         param = roslibpy.Param(self.ros, parameter_name)
-        srdf = await_callback(param.get)
+        srdf = param.get(timeout=TIMEOUT)
 
         if self.local_cache_enabled:
             _write_file(self._srdf_filename, srdf)
@@ -205,7 +205,7 @@ class RosFileServerLoader(object):
         if not use_local_file:
             service = roslibpy.Service(self.ros, '/file_server/get_file', 'file_server/GetBinaryFile')
             request = roslibpy.ServiceRequest(dict(name=url))
-            response = await_callback(service.call, 'callback', 'errback', request)
+            response = service.call(request, timeout=TIMEOUT)
 
             file_content = binascii.a2b_base64(response.data['value'])
 
@@ -234,6 +234,8 @@ def _dae_mesh_importer(filename, precision):
     dae = XML.from_file(filename)
     meshes = []
     visual_scenes = dae.root.find('library_visual_scenes')
+    materials = dae.root.find('library_materials')
+    effects = dae.root.find('library_effects')
 
     for geometry in dae.root.findall('.//geometry'):
         mesh_xml = geometry.find('mesh')
@@ -252,42 +254,65 @@ def _dae_mesh_importer(filename, precision):
                 M = M[0:4], M[4:8], M[8:12], M[12:16]
                 transform = Transformation.from_matrix(M)
 
-        triangle_set = mesh_xml.find('triangles')
-        triangle_set_data = triangle_set.find('p').text.split()
+        for triangle_set in mesh_xml.findall('triangles'):
+            triangle_set_data = triangle_set.find('p').text.split()
 
-        # Parse vertices
-        vertices_input = triangle_set.find('input[@semantic="VERTEX"]')
-        vertices_link = mesh_xml.find('vertices[@id="{}"]/input'.format(vertices_input.attrib['source'][1:]))
-        positions = mesh_xml.find('source[@id="{}"]/float_array'.format(vertices_link.attrib['source'][1:]))
-        positions = positions.text.split()
+            # Try to retrieve mesh colors
+            mesh_colors = {}
 
-        vertices = [[float(p) for p in positions[i:i + 3]] for i in range(0, len(positions), 3)]
+            if materials is not None and effects is not None:
+                try:
+                    instance_effect = None
+                    material_id = triangle_set.attrib.get('material')
 
-        # Parse faces
-        faces = [int(f) for f in triangle_set_data[::2]]  # Ignore normals (every second item is normal index)
-        faces = [faces[i:i + 3] for i in range(0, len(faces), 3)]
+                    if material_id is not None:
+                        instance_effect = materials.find('material[@id="{}"]/instance_effect'.format(material_id))
 
-        # Rebuild vertices and faces using the same logic that other importers
-        # use remapping everything based on a selected precision
-        index_key = OrderedDict()
-        vertex = OrderedDict()
+                    if instance_effect is not None:
+                        instance_effect_id = instance_effect.attrib['url'][1:]
+                        colors = effects.findall('effect[@id="{}"]/profile_COMMON/technique/phong/*/color'.format(instance_effect_id))
+                        for color_node in colors:
+                            rgba = [float(i) for i in color_node.text.split()]
+                            mesh_colors['mesh_color.{}'.format(color_node.attrib['sid'])] = rgba
+                except Exception:
+                    LOGGER.exception('Exception while loading materials, all materials of mesh file %s will be ignored ', filename)
 
-        for i, xyz in enumerate(vertices):
-            key = geometric_key(xyz, precision)
-            index_key[i] = key
-            vertex[key] = xyz
+            # Parse vertices
+            vertices_input = triangle_set.find('input[@semantic="VERTEX"]')
+            vertices_link = mesh_xml.find('vertices[@id="{}"]/input'.format(vertices_input.attrib['source'][1:]))
+            positions = mesh_xml.find('source[@id="{}"]/float_array'.format(vertices_link.attrib['source'][1:]))
+            positions = positions.text.split()
 
-        key_index = {key: index for index, key in enumerate(vertex)}
-        index_index = {index: key_index[key] for index, key in iter(index_key.items())}
-        vertices = [xyz for xyz in iter(vertex.values())]
-        faces = [[index_index[index] for index in face] for face in faces]
+            vertices = [[float(p) for p in positions[i:i + 3]] for i in range(0, len(positions), 3)]
 
-        mesh = Mesh.from_vertices_and_faces(vertices, faces)
+            # Parse faces
+            faces = [int(f) for f in triangle_set_data[::2]]  # Ignore normals (every second item is normal index)
+            faces = [faces[i:i + 3] for i in range(0, len(faces), 3)]
 
-        if transform:
-            mesh_transform(mesh, transform)
+            # Rebuild vertices and faces using the same logic that other importers
+            # use remapping everything based on a selected precision
+            index_key = OrderedDict()
+            vertex = OrderedDict()
 
-        meshes.append(mesh)
+            for i, xyz in enumerate(vertices):
+                key = geometric_key(xyz, precision)
+                index_key[i] = key
+                vertex[key] = xyz
+
+            key_index = {key: index for index, key in enumerate(vertex)}
+            index_index = {index: key_index[key] for index, key in iter(index_key.items())}
+            vertices = [xyz for xyz in iter(vertex.values())]
+            faces = [[index_index[index] for index in face] for face in faces]
+
+            mesh = Mesh.from_vertices_and_faces(vertices, faces)
+
+            if mesh_colors:
+                mesh.attributes.update(mesh_colors)
+
+            if transform:
+                mesh_transform(mesh, transform)
+
+            meshes.append(mesh)
 
     return meshes
 
